#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include "SystemEnum.h"

/*
  SystemEnum C++ header for enumerating handles, modules, processes and applications
  with callback functions.
  Some code based on SystemInfo header by Zoltan Csizmadia, zoltan_csizmadia@yahoo.com.

  Title:   SystemEnum C++
  Author:  Afrow UK
  Version: 0.2
  Date:    26th September 2007

  SystemInfo.cpp Rewritten:

    * MFC not used.
    * NtQueryObject freezing fixed.
    * Callbacks used rather than CList/CMap collections.
    * TerminateThread used in the case of emergencies, but memory leak avoided.
    * Back to basics: No classes.

  Changes:

    v0.2 - 26th September 2007
      * Added EnumApplications.
      * WS_CAPTION is now used rather than WS_SYSMENU to identify application windows.
      * EnumWindowCaptions parameter struct deprecated.
      * Removed GetProcessImageFileName dependency (only available on Win XP).

    v0.1 - 10th July 2007
      * First version.

*/

BOOL g_bFinishNow = FALSE;

// Get the NtDll and PSAPI function pointers.
BOOL SystemFuncInit()
{
  if (
    NtQuerySystemInformation  == NULL ||
    NtQueryObject             == NULL ||
    NtQueryInformationFile    == NULL ||
    EnumProcesses             == NULL ||
    EnumProcessModules        == NULL ||
    GetModuleFileNameEx       == NULL
     )
  {
    HINSTANCE hDll;

    hDll = GetModuleHandle(TEXT("ntdll.dll"));
    NtQuerySystemInformation  = (PNtQuerySystemInformation) GetProcAddress(hDll, "NtQuerySystemInformation");
    NtQueryObject             = (PNtQueryObject)            GetProcAddress(hDll, "NtQueryObject");
    NtQueryInformationFile    = (PNtQueryInformationFile)   GetProcAddress(hDll, "NtQueryInformationFile");

    hDll = LoadLibrary(TEXT("PSAPI.DLL"));
    EnumProcesses             = (PEnumProcesses)            GetProcAddress(hDll, "EnumProcesses");
    EnumProcessModules        = (PEnumProcessModules)       GetProcAddress(hDll, "EnumProcessModules");
    GetModuleFileNameEx       = (PGetModuleFileNameEx)      GetProcAddress(hDll, "GetModuleFileNameExA");
  }
  return 
    NtQuerySystemInformation  != NULL &&
    NtQueryObject             != NULL &&
    NtQueryInformationFile    != NULL &&
    EnumProcesses             != NULL &&
    EnumProcessModules        != NULL &&
    GetModuleFileNameEx       != NULL;
}

// Converts a unicode string buffer (2 bytes per character) to an ASCII string (1 byte per character).
void WCHAR2MB(PWSTR wstr, DWORD wstrlen, TCHAR *buff, DWORD bufflen)
{
  WideCharToMultiByte(
    CP_UTF8,
    0,
    wstr,
    wstrlen,
    buff,
    bufflen,
    NULL,
    NULL);
}

// Does a string end with a certain string?
BOOL EndsWith(TCHAR *in, const TCHAR *end)
{
  int inlen = lstrlen(in);
  int endlen = lstrlen(end);
  while (inlen > 0 && endlen > 0)
  {
    inlen--;
    endlen--;
    if (in[inlen] != end[endlen])
      return FALSE;
  }
  return TRUE;
}

// Gets the file name from a complete path.
void GetFileFromPath(TCHAR *in, int inlen, TCHAR *out, int outlen)
{
  int i = 0;
  while (in[i] != 0 && i<inlen)
    i++;
  while (in[i] != '\\' && i>0)
    i--;
  i++;
  for (int j=0; j<outlen; j++, i++)
  {
    out[j] = in[i];
    if (in[i] == 0)
      break;
  }
}

// Convert a device file path into a DOS file path.
BOOL GetFsFilePath(LPCTSTR lpDeviceFilePath, TCHAR *pszFilePath)
{
  TCHAR *pszDevicePath = (TCHAR*)LocalAlloc(LPTR, sizeof(TCHAR)*1024);
  TCHAR szDrive[3] = _T("A:");

  // Iterate through all drives.
  for (TCHAR chDrive = _T('A'); chDrive <= _T('Z'); chDrive++)
  {
    szDrive[0] = chDrive;

    // Query the device for the drive letter.
    if (QueryDosDevice(szDrive, pszDevicePath, sizeof(TCHAR)*1024) != 0)
    {
      // Network drive?
      if (_tcsnicmp(_T("\\Device\\LanmanRedirector\\"), pszDevicePath, 25) == 0)
      {
    	TCHAR chDriveLetter;
        DWORD dwParam;

        TCHAR *pszSharedPath = (TCHAR*)LocalAlloc(LPTR, sizeof(TCHAR)*1024);

        if (_stscanf(pszDevicePath, _T("\\Device\\LanmanRedirector\\;%c:%d\\%s"), &chDriveLetter, &dwParam, pszSharedPath) != 3)
        {
          LocalFree(pszSharedPath);
          continue;
        }

        _tcscpy(pszDevicePath, _T("\\Device\\LanmanRedirector\\"));
        _tcscat(pszDevicePath, pszSharedPath);

        LocalFree(pszSharedPath);
      }
      
      // Is this the drive letter we are looking for?
      int iDevicePathLen = _tcslen(pszDevicePath);
      if (_tcsnicmp(pszDevicePath, lpDeviceFilePath, iDevicePathLen) == 0)
      {
        _tcscpy(pszFilePath, szDrive);
        for (int i=2; ; i++, iDevicePathLen++)
        {
          pszFilePath[i] = lpDeviceFilePath[iDevicePathLen];
          if (pszFilePath[i] == 0)
            break;
        }

        LocalFree(pszDevicePath);

        return TRUE;
      }
    }
  }

  return FALSE;
}

// Open a process for handle duplication.
HANDLE OpenProcess(DWORD dwProcessId)
{
  return OpenProcess(PROCESS_DUP_HANDLE, TRUE, dwProcessId);
}

// Duplicate a handle.
HANDLE DuplicateHandle(HANDLE hProcess, HANDLE hRemote)
{
  HANDLE hDup = NULL;
  DuplicateHandle(hProcess, hRemote, GetCurrentProcess(), &hDup, 0, FALSE, DUPLICATE_SAME_ACCESS);
  return hDup;
}

// Thread to call NtQueryInformationFile with FileNameInformation.
DWORD WINAPI NtQueryInformationFileThread(void *ThreadParams)
{
  NTFUNC_THREAD_PARAMS *Params = (NTFUNC_THREAD_PARAMS*)ThreadParams;
  FILE_NAME_INFORMATION *FileInfo;
  DWORD iob[2];

  if (NtQueryInformationFile(Params->Handle, &iob, Params->Buffer, Params->BufferLength, FileNameInformation) != 0)
    return FALSE;

  FileInfo = (FILE_NAME_INFORMATION*)Params->Buffer;
  WCHAR2MB(FileInfo->FileName, FileInfo->FileNameLength, Params->OutBuffer, Params->OutBufferLength);

  return TRUE;
}

// Thread to call NtQueryObject with whatever.
DWORD WINAPI NtQueryObjectThread(void *ThreadParams)
{
  NTFUNC_THREAD_PARAMS *Params = (NTFUNC_THREAD_PARAMS*)ThreadParams;
  UNICODE_STRING *UnicodeString;

  if (NtQueryObject(Params->Handle, Params->ObjectInformationClass, Params->Buffer, Params->BufferLength, NULL) != 0)
    return FALSE;

  UnicodeString = (UNICODE_STRING*)Params->Buffer;
  WCHAR2MB(UnicodeString->Buffer, UnicodeString->MaximumLength, Params->OutBuffer, Params->OutBufferLength);

  return TRUE;
}

// Gets a full file path from a file handle.
BOOL GetHandleFilePath(HANDLE h, FILE_INFORMATION *File)
{
  HANDLE handle;
  HANDLE hRemoteProcess = NULL;
  BOOL bSuccess = FALSE;
  BOOL remote = File->ProcessId != GetCurrentProcessId();
  NTFUNC_THREAD_PARAMS ThreadParams;
  HANDLE hThread;
  int len = lstrlen(File->ProcessFullPath);

  // Skip Visual Studio's JIT debugger (Machine Debug Manager, mdm.exe).
  // It starts using 100% CPU when we check one of its handles :S
  if (EndsWith(File->ProcessFullPath, "\\mdm.exe"))
    goto cleanup;

  // Also skip services.exe.
  if (EndsWith(File->ProcessFullPath, "\\services.exe"))
    goto cleanup;

  if (remote)
  {
    hRemoteProcess = OpenProcess(File->ProcessId);
    if (hRemoteProcess == NULL)
      goto cleanup;
    handle = DuplicateHandle(hRemoteProcess, h);
  }
  else
    handle = h;

  ThreadParams.Handle = handle;
  ThreadParams.BufferLength = sizeof(TCHAR)*1024;
  ThreadParams.Buffer = (TCHAR*)LocalAlloc(LPTR, sizeof(TCHAR)*1024);
  ThreadParams.OutBufferLength = sizeof(TCHAR)*1024;
  ThreadParams.OutBuffer = (TCHAR*)LocalAlloc(LPTR, sizeof(TCHAR)*1024);

  // What type is it the handle?
  ThreadParams.ObjectInformationClass = ObjectTypeInformation;
  ZeroMemory(ThreadParams.Buffer, ThreadParams.BufferLength);
  ZeroMemory(ThreadParams.OutBuffer, ThreadParams.OutBufferLength);
  hThread = CreateThread(NULL, 0, NtQueryObjectThread, (void *)&ThreadParams, 0, NULL);
  // Terminate threads that are frozen.
  if (WaitForSingleObject(hThread, 200) == WAIT_TIMEOUT)
    TerminateThread(hThread, 0);
  CloseHandle(hThread);

  // Only want files and directories!
  if (lstrcmpi(ThreadParams.OutBuffer, "File") != 0 && lstrcmpi(ThreadParams.OutBuffer, "Directory") != 0)
    goto cleanup;

  // First try using NtQueryInformationFile which does not freeze when it doesn't get access
  // but instead returns an empty string as the file name.
  ZeroMemory(ThreadParams.Buffer, ThreadParams.BufferLength);
  ZeroMemory(ThreadParams.OutBuffer, ThreadParams.OutBufferLength);
  hThread = CreateThread(NULL, 0, NtQueryInformationFileThread, (void *)&ThreadParams, 0, NULL);
  // Terminate threads that are frozen.
  if (WaitForSingleObject(hThread, 200) == WAIT_TIMEOUT)
    TerminateThread(hThread, 0);
  CloseHandle(hThread);

  // The buffer is empty, do not continue.
  if (!*ThreadParams.OutBuffer)
    goto cleanup;

  // Get the full device path for the file now that we know we have access.
  ThreadParams.ObjectInformationClass = ObjectNameInformation;
  ZeroMemory(ThreadParams.Buffer, ThreadParams.BufferLength);
  ZeroMemory(ThreadParams.OutBuffer, ThreadParams.OutBufferLength);
  hThread = CreateThread(NULL, 0, NtQueryObjectThread, (void *)&ThreadParams, 0, NULL);
  // Terminate threads that are frozen.
  if (WaitForSingleObject(hThread, 200) == WAIT_TIMEOUT)
    TerminateThread(hThread, 0);
  CloseHandle(hThread);

  if (!*ThreadParams.OutBuffer)
    goto cleanup;

  if (GetFsFilePath(ThreadParams.OutBuffer, File->FullPath))
    bSuccess = TRUE;

cleanup:
  if (remote)
  {
    if (hRemoteProcess != NULL)
      CloseHandle(hRemoteProcess);

    if (handle != NULL)
      CloseHandle(handle);
  }

  if (ThreadParams.Buffer)
    LocalFree(ThreadParams.Buffer);

  if (ThreadParams.OutBuffer)
    LocalFree(ThreadParams.OutBuffer);

  return bSuccess;
}

// Get process window captions.
BOOL CALLBACK EnumWindowCaptionsProc(HWND hWnd, LPARAM lParam)
{
  FILE_INFORMATION *File = (FILE_INFORMATION*)lParam;
  DWORD dwProcessId = 0;
  GetWindowThreadProcessId(hWnd, &dwProcessId);

  if (dwProcessId == File->ProcessId)
  {
    // Check if the window has a system menu.
    DWORD dwStyle = GetWindowLongPtr(hWnd, GWL_STYLE);
    File->ProcessHWND = hWnd;
    if (dwStyle & WS_CAPTION)
    {
      GetWindowText(hWnd, File->ProcessCaption, sizeof(File->ProcessCaption));
      if (*File->ProcessCaption)
        return FALSE;
    }
  }

  return TRUE;
}

BOOL WINAPI EnumSystemHandles(ENUM_LOCKED_FILES lpEnumLockedFiles, LPARAM lParam)
{
  FILE_INFORMATION File;
  DWORD dwProcessIdLast = -1;
  DWORD size;

  if (!SystemFuncInit())
    return TRUE;
  
  // Find the size to allocate.
  SYSTEM_HANDLE_INFORMATION *pSysHandleInformation = (SYSTEM_HANDLE_INFORMATION*)LocalAlloc(LPTR, sizeof(SYSTEM_HANDLE_INFORMATION));
  if (pSysHandleInformation == NULL)
    return TRUE;
  NtQuerySystemInformation(SystemHandleInformation, pSysHandleInformation, sizeof(SYSTEM_HANDLE_INFORMATION), &size);
  LocalFree(pSysHandleInformation);

  // Allocate required memory.
  pSysHandleInformation = (SYSTEM_HANDLE_INFORMATION*)LocalAlloc(LPTR, size);
  if (pSysHandleInformation == NULL)
    return TRUE;

  // Query the objects (system wide).
  if (NtQuerySystemInformation(SystemHandleInformation, pSysHandleInformation, size, NULL) != 0)
  {
    LocalFree(pSysHandleInformation);
    return TRUE;
  }

  File.TotalFiles = pSysHandleInformation->Count;

  // Iterating through the objects.
  for (DWORD i=0; i<pSysHandleInformation->Count; i++)
  {

    File.FileNumber = i+1;
    File.ProcessId = pSysHandleInformation->Handles[i].ProcessId;

    // New process Id to get the image file name of.
    if (File.ProcessId != dwProcessIdLast)
    {
      HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, File.ProcessId);

      if (hProcess != NULL)
      {
        // Get the process image file name.
        GetModuleFileNameEx(hProcess, NULL, File.ProcessFullPath, MAX_PATH);
        CloseHandle(hProcess);
      }
      dwProcessIdLast = File.ProcessId;
    }

    // Get the file path of the handle.
    if (GetHandleFilePath((HANDLE)pSysHandleInformation->Handles[i].HandleValue, &File))
    {
      // Get process window caption.
      ZeroMemory(File.ProcessCaption, sizeof(File.ProcessCaption));
      File.ProcessHWND = NULL;
      EnumWindows(EnumWindowCaptionsProc, (LPARAM)&File);

      // Call the callback.
      if (!lpEnumLockedFiles(File, lParam))
      {
        LocalFree(pSysHandleInformation);
        return FALSE;
      }
    }

    // Exit right now?
    if (g_bFinishNow)
    {
      LocalFree(pSysHandleInformation);
      return FALSE;
    }
  }

  LocalFree(pSysHandleInformation);
  return TRUE;
}

BOOL WINAPI EnumSystemModules(ENUM_LOCKED_FILES lpEnumLockedFiles, LPARAM lParam)
{
  DWORD size = 128;

  if (!SystemFuncInit())
    return TRUE;

  DWORD *dwProcessIds = (DWORD*)LocalAlloc(LPTR, sizeof(DWORD)*size);
  
  if (EnumProcesses(dwProcessIds, sizeof(DWORD)*size, &size) == 0)
  {
    LocalFree(dwProcessIds);
    return TRUE;
  }

  for (DWORD i=0; i<size / sizeof(DWORD); i++)
  {
    FILE_INFORMATION File;
    DWORD size2 = 128;

    // Open the process to read its modules list.
    HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dwProcessIds[i]);

    // Get the number of modules.
    if (hProcess == NULL)
      continue;

    // Get the process image file name.
    GetModuleFileNameEx(hProcess, NULL, File.ProcessFullPath, MAX_PATH);

    File.FileNumber = i+1;
    File.TotalFiles = int(size / sizeof(DWORD));

    HMODULE *hModules = (HMODULE*)LocalAlloc(LPTR, sizeof(HMODULE)*size2);

    // Get a list of process modules.
    if (EnumProcessModules(hProcess, hModules, sizeof(HMODULE)*size2, &size2) == 0)
    {
      CloseHandle(hProcess);
      LocalFree(hModules);
      continue;
    }
  
    for (DWORD j=0; j<size2 / sizeof(HMODULE); j++)
    {
      File.ProcessId = dwProcessIds[i];

      // Get the file path of the module.
      if (GetModuleFileNameEx(hProcess, hModules[j], File.FullPath, MAX_PATH))
      {
        // Get process window caption.
        ZeroMemory(File.ProcessCaption, sizeof(File.ProcessCaption));
        File.ProcessHWND = NULL;
        EnumWindows(EnumWindowCaptionsProc, (LPARAM)&File);

        if (!lpEnumLockedFiles(File, lParam))
        {
          CloseHandle(hProcess);
          LocalFree(hModules);
          LocalFree(dwProcessIds);
          return FALSE;
        }
      }

      // Exit right now.
      if (g_bFinishNow)
      {
        CloseHandle(hProcess);
        LocalFree(hModules);
        LocalFree(dwProcessIds);
        return FALSE;
      }
    }

    CloseHandle(hProcess);
    LocalFree(hModules);

    // Exit right now.
    if (g_bFinishNow)
    {
      LocalFree(dwProcessIds);
      return FALSE;
    }
  }

  LocalFree(dwProcessIds);
  return TRUE;
}

BOOL WINAPI EnumProcessIds(ENUM_PROCESS_IDS lpEnumProcessIds, LPARAM lParam)
{
  DWORD size = 128;

  if (!SystemFuncInit())
    return TRUE;

  DWORD *dwProcessIds = (DWORD*)LocalAlloc(LPTR, sizeof(DWORD)*size);

  // Get a list of process ids.
  if (EnumProcesses(dwProcessIds, sizeof(DWORD)*size, &size) == 0)
  {
    LocalFree(dwProcessIds);
    return TRUE;
  }

  // Loop through the process ids.
  for (DWORD i=0; i<size / sizeof(DWORD); i++)
  {
    if (!lpEnumProcessIds(dwProcessIds[i], lParam))
    {
      LocalFree(dwProcessIds);
      return FALSE;
    }
  }

  LocalFree(dwProcessIds);
  return TRUE;
}

BOOL CALLBACK EnumApplicationsCountProc(HWND hWnd, LPARAM lParam)
{
  DWORD dwStyle = GetWindowLongPtr(hWnd, GWL_STYLE);
  if (dwStyle & WS_VISIBLE)
    (int*)lParam++;
  return TRUE;
}

// Get windows.
BOOL CALLBACK EnumApplicationsProc(HWND hWnd, LPARAM lParam)
{
  ENUM_APPLICATIONS_PARAMS *Params = (ENUM_APPLICATIONS_PARAMS*)lParam;
  FILE_INFORMATION File;
  DWORD dwProcessId = 0;

  DWORD dwStyle = GetWindowLongPtr(hWnd, GWL_STYLE);
  if (!(dwStyle & WS_VISIBLE))
    return TRUE;

  ZeroMemory(File.ProcessCaption, sizeof(File.ProcessCaption));
  GetWindowText(hWnd, File.ProcessCaption, sizeof(File.ProcessCaption));
  if (!*File.ProcessCaption)
    return TRUE;

  // Open the process to get further information.
  GetWindowThreadProcessId(hWnd, &dwProcessId);
  HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dwProcessId);

  // Get the number of modules.
  if (hProcess == NULL)
    return TRUE;

  // Get the process image file name.
  GetModuleFileNameEx(hProcess, NULL, File.ProcessFullPath, MAX_PATH);
  
  CloseHandle(hProcess);

  File.ProcessId = dwProcessId;
  File.FileNumber = Params->number++;
  File.ProcessHWND = hWnd;

  Params->lpEnumApplications(File, Params->lParam);

  return TRUE;
}

BOOL WINAPI EnumApplications(ENUM_APPLICATIONS lpEnumApplications, LPARAM lParam)
{
  ENUM_APPLICATIONS_PARAMS Params;
  Params.number = 0;

  if (!SystemFuncInit())
    return TRUE;

  EnumWindows(EnumApplicationsCountProc, (LPARAM)&Params.count);

  Params.lParam = lParam;
  Params.lpEnumApplications = lpEnumApplications;
  EnumWindows(EnumApplicationsProc, (LPARAM)&Params);

  return TRUE;
}

// Finish the enumeration functions ASAP.
void FinishEnumeratingNow()
{
  g_bFinishNow = TRUE;
}
